$fn = 15;

// X-Y spacing for Atomic Pi
standoff_spacing_x = 121.9;
standoff_spacing_y = 91.9;

power_conn_dia = 7.94;
power_conn_radius = (power_conn_dia *1.1)/2.0;
usb_hub_dim = 45.25 + 0.1; // actual is 45, add fudge factor

api_y = 105 + 55;
api_x = standoff_spacing_x + 45;

wall_thickness = 2;
base_thickness = 3;

outer_x_dimension = wall_thickness * 2 + api_x;
outer_y_dimension = wall_thickness * 2 + api_y;

api_heat_sink_height = 36;
standoff_height = 18.5;
interior_height = standoff_height + api_heat_sink_height+ 5;

arduino_board_height = 53.29;
arduino_board_width = 36.1;

// Note: I just eyeballed this
buck_standoffs_vec =[42,114,0];

lid_hole_radius = 4/2.0;
lid_hole_dim = 3.5;
lid_hole_depth = 5.0;

x_board_padding = lid_hole_dim + 19;
y_board_padding = 2;

function hadamard(a,b) = !(len(a)>0) ? a*b : [ for(i = [0:len(a)-1]) hadamard(a[i],b[i]) ];

build_box = 0;

// a lid
translate([-200,0,63]){
  difference(){
    cube([outer_x_dimension, outer_y_dimension, base_thickness]);
    
    lid_hole_offset = wall_thickness + lid_hole_dim;
    lid_fastener_hole_radius = lid_hole_radius * 1.33;
    union(){
      translate([lid_hole_offset, lid_hole_offset, -1]){
        cylinder(r = lid_fastener_hole_radius, h = base_thickness * 2);
      }
      
      translate([outer_x_dimension - lid_hole_offset, lid_hole_offset, -1]){
        cylinder(r = lid_fastener_hole_radius, h = base_thickness * 2);
      }
      translate([outer_x_dimension - lid_hole_offset, outer_y_dimension - lid_hole_offset, -1]){
        cylinder(r = lid_fastener_hole_radius, h = base_thickness * 2);
      }
      
      translate([lid_hole_offset, outer_y_dimension - lid_hole_offset, -1]){
        cylinder(r = lid_fastener_hole_radius, h = base_thickness * 2);
      }
      
      lid_vent_size = 10;
      translate([wall_thickness + x_board_padding, wall_thickness + y_board_padding, -1]){
        for(x_disp = [1:8]){
          for(y_disp = [1:6]){
            translate([x_disp * (lid_vent_size + wall_thickness), y_disp * (lid_vent_size +wall_thickness), 0]){
              cube([lid_vent_size, lid_vent_size, base_thickness * 2]);
            }
          }
        }
      }
    }
  }
}

x_y_disp = [
  [0,0,0],
  [1,0,0],
  [0,1,0],
  [1,1,0]
  ];

ssd_y_overall = 69.85;
ssd_extra_depth_min = 58;
ssd_extra_depth = api_y - (ssd_y_overall + ssd_extra_depth_min) + 15;
ssd_standoff_dia = 7;
ssd_extra_buffer = 4;
ssd_extra_width = ssd_y_overall + ssd_standoff_dia + ssd_extra_buffer;

ssd_y_spacing = 61.72;
ssd_x_spacing = 90.60 - 14;
ssd_x_overall = 100.4;
ssd_hole_radius = 3.2 / 2.0;
ssd_x_buffer = 32;
ssd_x_padding = ssd_x_spacing + ssd_x_buffer;
ssd_y_padding = -1 * (ssd_extra_depth - 4) + ssd_standoff_dia;
ssd_padding = [ssd_x_padding, ssd_y_padding,0];
ssd_rotate_vec = [0,0,90];
ssd_extra_height = 14;
// cap for SSD thing
translate([ssd_x_buffer + ssd_standoff_dia  - (ssd_extra_buffer/2.0) - wall_thickness, -1 * (2*wall_thickness + ssd_extra_depth),ssd_extra_height]){  
  difference(){
    color("red") cube([ssd_extra_width + wall_thickness  *2, ssd_extra_depth + wall_thickness * 1, base_thickness + wall_thickness]);
    translate([wall_thickness, wall_thickness, -1]){
      cube([ssd_extra_width, ssd_extra_depth + 10, base_thickness + 1]);
    }
  }
}

module ssd_cylinders(r, h){  
  for(disp_vec = x_y_disp){
    translate(hadamard(disp_vec, [ssd_x_spacing,ssd_y_spacing, 0])){
      cylinder(r = r, h = h);
    }
  }
  
}

//base 
difference(){  
  union(){
    cube([outer_x_dimension, outer_y_dimension, base_thickness]);
    translate([ssd_x_buffer + ssd_standoff_dia  - (ssd_extra_buffer/2.0) ,-1 * (ssd_extra_depth + wall_thickness),0]){
        cube([ssd_extra_width,(ssd_extra_depth + wall_thickness),base_thickness]);
      }
  }
  
  union(){
    // holes for SSD screws
    translate(ssd_padding + [0,0,0]){
      rotate(ssd_rotate_vec){
        ssd_cylinders(ssd_hole_radius + 3, base_thickness * 10);
      }
    };
    translate(buck_standoffs_vec){
      translate([-29,8,-1]){
        cube([28,25,base_thickness * 2]);
      }
    }
  }
}

buck_y_dim = 30.66;
buck_x_dim = 45;
buck_hole_radius = 3.6/2.0;
buck_standoff_hole_radius = 3/2.0;
buck_standoff_radius = buck_standoff_hole_radius + 1.75;
buck_spacing_y = 23;
buck_spacing_x = 35; // Measured from the first set of holes!
buck_mounting_height = 3.5;
buck_origin_hole_offset = 4.2;
module buck_standoffs(){
  translate([buck_standoff_radius, buck_standoff_radius, 0]){
    for(disp_vec = [
      [0,0,0],
      [0,1,0]
      ]){
      translate(hadamard(disp_vec, [buck_spacing_x,buck_spacing_y, 0])){
        difference(){
          cylinder(r = buck_standoff_radius, h = buck_mounting_height);
          translate([0,0,1]){
            cylinder(r = buck_standoff_hole_radius, h = buck_mounting_height - 1);
          }
        }
      }
    }
    
    translate([buck_spacing_x,buck_y_dim/2.0 - buck_origin_hole_offset,0]){
        difference(){
          cylinder(r = buck_standoff_radius, h = buck_mounting_height);
          translate([0,0,1]){
            cylinder(r = buck_standoff_hole_radius, h = buck_mounting_height - 1);
          }
        }
    }
  }
}

buck_vent_hole_radius = 5;
buck_y_padding = 3;
module buck_base(){
  difference(){
    cube([buck_x_dim, buck_y_padding + buck_y_dim, base_thickness]);  
    union(){
      translate([base_thickness + buck_vent_hole_radius,  buck_y_dim/2.0, 0]){
        cylinder(r = buck_vent_hole_radius, h =base_thickness +1);
        
        translate([buck_vent_hole_radius * 1.33 + base_thickness,buck_vent_hole_radius  *1.25,0]){
          cylinder(r = buck_vent_hole_radius, h =base_thickness +1);  
        }
          
        translate([buck_vent_hole_radius * 1.33 + base_thickness,-1 * (buck_vent_hole_radius *1.25),0]){
          cylinder(r = buck_vent_hole_radius, h =base_thickness +1);  
        }
        
        translate([buck_vent_hole_radius * 3.33 + base_thickness,0,0]){
          cylinder(r = 1.125 * buck_vent_hole_radius, h =base_thickness +1);  
        }
      }      
    }
  }
  
  translate([-1 * base_thickness, 0, 0]){
    //cube([base_thickness, buck_y_dim + buck_y_padding, base_thickness * 1.5]);
  }
  
  translate([buck_x_dim, 0, 0]){
    //cube([base_thickness, buck_y_dim + buck_y_padding, base_thickness * 1.5]);
  }
  
}

module lid_mount(){
  color("pink"){
    difference(){
      union(){
        cube([ lid_hole_dim,lid_hole_dim * 2, interior_height]);
        cube([ lid_hole_dim * 2,lid_hole_dim, interior_height]);
        
        translate([lid_hole_dim, lid_hole_dim, 0]){
          cylinder(r = lid_hole_dim, h = interior_height);
        }
      }
      
      translate([lid_hole_dim, lid_hole_dim, interior_height - lid_hole_depth]){
        cylinder(r = lid_hole_radius, h = lid_hole_depth + 1);
      }
      
    }
  }
}

module long_wall(){
  cube([outer_x_dimension, wall_thickness, interior_height]);
};

module short_wall() {
  cube([wall_thickness, outer_y_dimension, interior_height]);  
}

standoff_radius = 4.0;
standoff_hole_radius = 3/2.0;

module standoff(support_dimension, support_rotation){
  translate([standoff_radius, standoff_radius,0]){
  
    difference(){
      union(){
        cylinder(r = standoff_radius, h = standoff_height);
        cylinder(r1 = standoff_radius + 0.75, r2 = standoff_radius, h = standoff_height * 0.25);
        rotate(support_rotation){
          translate([-1 * base_thickness/2.0, -1 * (support_dimension + standoff_radius), 0]){
            cube([wall_thickness,support_dimension, standoff_height - 3]);
          }
        }
      }
      translate([0,0,standoff_height - 10]){
        cylinder(r = standoff_hole_radius, h = 10);
      }
    }        
  }
}

  
//USB hub mount
usb_hub_x_offset = api_x - usb_hub_dim - 33;
usb_hub_perimeter = 3;
usb_hub_height = 13.9;
usb_hub_post_dim = 6;
usb_hub_post_hole_radius = 2.8/2.0;
usb_hub_post_hole_depth = 5;
module usb_hub_post(){
  difference(){
    cube([usb_hub_post_dim, usb_hub_post_dim, usb_hub_height]);
    
    translate([usb_hub_post_dim/2,usb_hub_post_dim/2,usb_hub_height - usb_hub_post_hole_depth]){
      cylinder(r= usb_hub_post_hole_radius, h = usb_hub_post_hole_depth + 1);
    }
  }
};


function dim_offset_if_even(i, dim) = (i % 2 == 0 ? dim : dim * 0.5);

buck2_y=20.638;
buck2_x=11.11;
buck2_wall = 2;
buck2_offset_x = 3;
buck2_offset_y = 68;
translate([0,0,base_thickness]){  
  
  // buck2 base - 
  buck2_base_outer_x = buck2_x + buck2_wall*2;
  buck2_base_outer_y = buck2_y + buck2_wall*2;
  
  color("grey") translate([buck2_offset_x,buck2_offset_y,0]){
    difference(){
      cube([buck2_base_outer_x,buck2_base_outer_y, buck2_wall]);
      translate([buck2_wall, buck2_wall,0]){
        cube([buck2_x,buck2_y,buck2_wall + 1]);
      }
    }
  }

// usb hub mount
  /**
  translate([usb_hub_x_offset, api_y - usb_hub_dim - wall_thickness, 0]){
    color("grey"){
      difference(){
        cube([usb_hub_dim + usb_hub_perimeter*2,usb_hub_dim + usb_hub_perimeter*2, 2.5]);
        union(){
          translate([usb_hub_perimeter, usb_hub_perimeter, 0]){
            cube([usb_hub_dim,usb_hub_dim, 3]);
          }
          translate([0,usb_hub_dim + usb_hub_perimeter,0]){
            cube([usb_hub_dim + usb_hub_perimeter*2,usb_hub_dim + usb_hub_perimeter, 3 ]);
            
          }
        }
      }
    }
    
    color("pink"){
      translate([-1 * usb_hub_post_dim,0,0]){
        for(i = x_y_disp){
          translate(hadamard(i,[usb_hub_dim + usb_hub_perimeter*2 + usb_hub_post_dim, usb_hub_dim - wall_thickness,0])){
            usb_hub_post();            
          }
        }
      } 
    }
  }
  **/
  
  translate([wall_thickness, wall_thickness, 0]){
   lid_mount();     
  }
  
  translate([wall_thickness + api_x, wall_thickness + api_y,0]){
    rotate([0,0,180]){
      lid_mount();
    }
  }
  translate([wall_thickness + api_x, wall_thickness,0]){
    rotate([0,0,90]){
      lid_mount();
    }
  }  
  translate([wall_thickness, wall_thickness + api_y,0]){
    rotate([0,0,-90]){
      lid_mount();
    }
  }
     
  // buck converter mount
  translate(buck_standoffs_vec){
    rotate([0,0,90], center = true){
      buck_standoffs();
    }
  }


  
  // supports to space SSD off the bottom
  translate(ssd_padding){
    rotate(ssd_rotate_vec){
      
      difference(){
        color("pink") ssd_cylinders(ssd_standoff_dia, 1.75);
        ssd_cylinders(ssd_hole_radius, 8);  
      }
      //end support for SSD adapter
      translate([ssd_x_overall,0,0]){
        color("green") cube([12,ssd_y_spacing,1.5]);
      }
    }          
  }

  hdmi_offset = 102;
  hdmi_height = 6;
  hdmi_width = 15;
  hdmi_margin_y = 6.5;
  sma_dia = 6.35/2.0;
  sma_height = 11;
  hdmi_margin_z= 5;
  sma_radius_addl = 0.11 * sma_dia;
  
 
  difference(){    
    union(){
      long_wall();
      translate([ssd_x_buffer + ssd_standoff_dia  - (ssd_extra_buffer/2.0) ,-1 * (ssd_extra_depth + wall_thickness) ,0]){
        
        // create a cube then cut it out
        difference(){
          cube([ssd_extra_width,ssd_extra_depth + wall_thickness,ssd_extra_height]);
          translate([wall_thickness, wall_thickness, 0]){
            cube([ssd_extra_width - wall_thickness * 2, ssd_extra_depth , ssd_extra_height]);
          }
        }
      }
    }
    
    base_x_for_connectors = wall_thickness + x_board_padding;
    union(){
      
      // Cut out hole for SSD
      translate([ssd_x_buffer + ssd_standoff_dia - (ssd_extra_buffer/2.0) + wall_thickness ,0, 0]){
        cube([ssd_extra_width -(wall_thickness * 2),wall_thickness  * 2,ssd_extra_height]);
      }
      
      translate([lid_hole_dim+ 13,wall_thickness * 1.1,8]){
        rotate([90,0,0]){
          cylinder(r = power_conn_radius, h = 2 * wall_thickness);
        }
      }
      
      translate([base_x_for_connectors + hdmi_offset - hdmi_margin_y, 0,standoff_height - hdmi_margin_z/1.5]){
        cube([hdmi_width + hdmi_margin_y * 2,wall_thickness, hdmi_height + hdmi_margin_z * 2]);
      }
      translate([20, wall_thickness*2, standoff_height + 33]){
        rotate([90, 0, 0]){
          cylinder(r = sma_dia + sma_radius_addl, h = wall_thickness + 2);
        }        
        translate([api_x - 40,0,0]){
          rotate([90, 0, 0]){
            cylinder(r = sma_dia + sma_radius_addl, h = wall_thickness + 2);
          }   
        }
      }
    }
  }
     
  color("purple") {
    hole_dim = 5;
    difference(){
      short_wall();
      translate([-1, wall_thickness/2.0 ,base_thickness*0.5]){
        union(){          
            for(y = [1:15]){
              for(x = [0:9]){              
                
                translate([0, dim_offset_if_even(x,  (hole_dim + wall_thickness * 2)) + (hole_dim + wall_thickness * 2)*y , x * (hole_dim * wall_thickness * 0.5)]){
                rotate([45,0,0]){
                  if(!(y == 10 && x<3) && !(y == 11 && x==1)){
                    cube([wall_thickness * 2, hole_dim, hole_dim]);
                  }
                }
              }
            }
          }
        }
      }
    }
    
  }
  
  switch_radius = 5.95/2.0;
  arduino_board_offset = 119;
  arduino_board_thickness = 1.6;
  arduino_standoff_height = 4;
  arduino_board_depth = arduino_standoff_height + arduino_standoff_height;   
  arduino_board_padding = 0.5;
  arduino_hole_x = arduino_board_width  + wall_thickness * 2 + arduino_board_padding * 2;
  arduino_hole_y = arduino_board_depth + wall_thickness * 2 + arduino_board_padding * 2;
  arduino_hole_z = arduino_board_height + wall_thickness + arduino_board_padding * 2;
  
  arduino_hole_radius = 3.175/2.0;
  arduino_screw_hole_radius =arduino_hole_radius - 0.15;
  arduino_mount_spacing = 44.9 - (arduino_hole_radius*2);
  arduino_lower_mount_offset = 5.2;
  arduino_support_strip = 7;
  
  
  
   translate([0,outer_y_dimension - wall_thickness,0]){
    difference(){
      color ("red") long_wall();                
      union(){    
        // hole for SMA connector for BT
        translate([25, wall_thickness*2, standoff_height + 33]){
          rotate([90, 0, 0]){
            cylinder(r = sma_dia + sma_radius_addl, h = wall_thickness + 2);
          }                
        }
        usb_hub_hole = 39;
        /**
        translate([usb_hub_x_offset + usb_hub_perimeter + usb_hub_dim/2.0 - usb_hub_hole/2.0,-1,2]){
          cube([39,wall_thickness  * 2,11]);
          
        }  **/      
        // hole for power switch
        // TODO bars to hold switch in place
        switch_offset_x = lid_hole_dim + wall_thickness + (4 * switch_radius);
        translate([switch_offset_x, wall_thickness * 2, 30]){
          for(i = [0:1]){
            translate([i * 15, 0, 0]){
              rotate([90, 0, 0]){
                cylinder(r = switch_radius *1.11, h = wall_thickness + 2);
              }                          
            }
          }
        }
        
        // hole for arduino nano board        
        translate([arduino_board_offset,-1,0]){
          cube([ arduino_hole_x, wall_thickness  *2, arduino_hole_z]);
        }
        
        // hole for USB connectors
        usb_conn_width = 22;
        usb_conn_height = 28.5;
        usb_conn_z_padding = 10;
        usb_conn_hole_radius = 5/2.0;
        usb_conn_hole_spacing = 41.33 - usb_conn_hole_radius*2;
        usb_conn_lower_pad = 8.25;
        
        translate([80,-1,usb_conn_z_padding]){
          translate([usb_conn_width/2.0, wall_thickness*2, usb_conn_lower_pad/2.0]){
            rotate([90,0,0]){
              cylinder(r=usb_conn_hole_radius, h = wall_thickness *2);          
            }
            translate([0,0,usb_conn_hole_spacing]){
              rotate([90,0,0]){
                cylinder(r=usb_conn_hole_radius, h = wall_thickness *2);          
              }
            }
          }
          translate([0,0,usb_conn_lower_pad]){          
            cube([usb_conn_width, wall_thickness *2, usb_conn_height]);
          }  
        }
        
        
    }
    }    
    
    color("orange") translate([arduino_board_offset,-1 * arduino_hole_y, 0]){
      difference(){
        cube([ arduino_hole_x, arduino_hole_y + wall_thickness, arduino_hole_z]);
        union(){
          translate([wall_thickness, -1, 0]){
            cube([arduino_board_width + arduino_board_padding * 2, 2 * wall_thickness + arduino_hole_y + arduino_board_padding* 2, arduino_board_height + arduino_board_padding * 2]);
            
            
          }
          translate([wall_thickness * 1.5, -1, arduino_board_height * (2/3.0)]){
            //cube([arduino_board_width, wall_thickness *2.0, arduino_board_height /3.0]);            
          }
        }
      }
      
      //small cubes down in the bottom corners to support the board
      translate([wall_thickness  ,0,0]){
        cube([1.5, arduino_hole_y + arduino_board_padding * 2 - arduino_board_thickness, 4]);
        translate([arduino_board_width,0,0]){
          cube([1.5, arduino_hole_y + arduino_board_padding * 2 - arduino_board_thickness, 4]);
        }
      }
      
      // center support & holes
      translate([arduino_board_width/2.0 - arduino_support_strip/2.0 + wall_thickness  + arduino_board_padding ,0,0]){        
        difference(){
          
          cube([arduino_support_strip, arduino_hole_y + arduino_board_padding * 2 - arduino_board_thickness, arduino_board_height + arduino_board_padding * 2]);
            
            
          
          union(){                        
            translate([arduino_support_strip/2.0,arduino_board_depth * 2 + 1,arduino_lower_mount_offset + arduino_board_padding]){
              rotate([90.0,0]){
                cylinder(r = arduino_screw_hole_radius, h =arduino_board_height *2);
              }
              translate([0,0, arduino_mount_spacing]){
                rotate([90.0,0]){
                  cylinder(r = arduino_screw_hole_radius, h =arduino_board_height *2);
                }  
              }
              
            }            
          }
        }
      }
    }
    
    
    
  }
  
  
  
  color("green") translate([outer_x_dimension - wall_thickness ,0, 0]){
    short_wall();
  }
  

  
  translate([x_board_padding, y_board_padding + wall_thickness, 0,]){
    for (disp_vec = x_y_disp){
      translate(hadamard(disp_vec, [standoff_spacing_x,standoff_spacing_y, 0])){
        if(disp_vec == [0,1,0]){
          standoff(x_board_padding - 1,[0,0,-90]);        
        }else if(disp_vec == [1,1,0]){
          standoff(17.5,[0,0,90]);        
        }else{
          standoff(y_board_padding + 1,[0,0,0]);        
        }
      }
    };
  
  }
  
};
